/*
 * SceneTest.java
 */

package de.ba_ravensburg.gdv.model;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Iterator;

import junit.framework.TestCase;

/**
 * @author Ansgar Wiechers
 */
public class SceneTest extends TestCase {
  
  private final double PRECISION = 0.0001;
  private final int X = Scene.X;
  private final int Y = Scene.Y;
  private final int Z = Scene.Z;
  private final int LEFTRIGHT = Scene.LEFTRIGHT;
  private final int UPDOWN = Scene.UPDOWN;

  /**
   * Constructor for SceneTest.
   * @param arg0
   */
  public SceneTest(String arg0) {
    super(arg0);
  }

  /*
   * Test for constructor Scene().
   */
  public void testScene() {
    double[] actualPos;
    double[] actualDir;
    Scene scene = new Scene();
    Field pos, dir;
    
    Class c = scene.getClass();
    
    try{
      pos = c.getDeclaredField("cameraPos"); //$NON-NLS-1$
      pos.setAccessible(true);
      
      actualPos = (double[])pos.get(scene);
      assertEquals(Scene.DEFAULT_POSITION[X], actualPos[X], PRECISION);
      assertEquals(Scene.DEFAULT_POSITION[Y], actualPos[Y], PRECISION);
      assertEquals(Scene.DEFAULT_POSITION[Z], actualPos[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraX"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_X[X], actualDir[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_X[Y], actualDir[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_X[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraY"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_Y[X], actualDir[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Y[Y], actualDir[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Y[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraZ"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_Z[X], actualDir[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Z[Y], actualDir[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Z[Z], actualDir[Z], PRECISION);
    } catch (Exception e){
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
  }

  /*
   * Test for constructor Scene(double, double, double).
   */
  public void testScenedoubledoubledouble() {
    double[] actualPos;
    double[] actualDir;
    double[] expectedPos = {1.0, 1.0, 1.0};
    Scene scene = new Scene(expectedPos[X], expectedPos[Y], expectedPos[Z]);
    Field pos, dir;
    
    Class c = scene.getClass();
    
    try{
      pos = c.getDeclaredField("cameraPos"); //$NON-NLS-1$
      pos.setAccessible(true);
      
      actualPos = (double[])pos.get(scene);
      assertEquals(expectedPos[X], actualPos[X], PRECISION);
      assertEquals(expectedPos[Y], actualPos[Y], PRECISION);
      assertEquals(expectedPos[Z], actualPos[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraX"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_X[X], actualDir[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_X[Y], actualDir[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_X[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraY"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_Y[X], actualDir[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Y[Y], actualDir[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Y[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraZ"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_Z[X], actualDir[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Z[Y], actualDir[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Z[Z], actualDir[Z], PRECISION);
    } catch (Exception e){
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
  }

  /*
   * Test for constructor Scene(double, double, double, double, double, double).
   */
  public void testScenedoubledoubledoubledoubledoubledouble() {
    double[] actualPos;
    double[] actualDir;
    double[] expectedPos = {1.0, 1.0, 1.0};
    double[] expectedDirX1 = {0.0, -1.0, 0.0};
    double[] expectedDirY1 = {1.0, 0.0, 0.0};
    double[] expectedDirZ1 = {0.0, 0.0, 1.0};
    double[] expectedDirX2 = {1.0, 0.0, 0.0};
    double[] expectedDirY2 = {0.0, 0.0, -1.0};
    double[] expectedDirZ2 = {0.0, 1.0, 0.0};
    Scene scene1 = new Scene(expectedPos[X], expectedPos[Y], expectedPos[Z],
                             expectedDirY1[X], expectedDirY1[Y], expectedDirY1[Z]);
    Scene scene2 = new Scene(expectedPos[X], expectedPos[Y], expectedPos[Z],
                             expectedDirY2[X], expectedDirY2[Y], expectedDirY2[Z]);
    Field pos, dir;
    
    Class c1 = scene1.getClass();
    Class c2 = scene2.getClass();
    
    try {
      /*
       * Scene turned 90 to the right.
       */
      pos = c1.getDeclaredField("cameraPos"); //$NON-NLS-1$
      pos.setAccessible(true);
      
      actualPos = (double[])pos.get(scene1);
      assertEquals(expectedPos[X], actualPos[X], PRECISION);
      assertEquals(expectedPos[Y], actualPos[Y], PRECISION);
      assertEquals(expectedPos[Z], actualPos[Z], PRECISION);
      
      dir = c1.getDeclaredField("cameraX"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene1);
      assertEquals(expectedDirX1[X], actualDir[X], PRECISION);
      assertEquals(expectedDirX1[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirX1[Z], actualDir[Z], PRECISION);
      
      dir = c1.getDeclaredField("cameraY"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene1);
      assertEquals(expectedDirY1[X], actualDir[X], PRECISION);
      assertEquals(expectedDirY1[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirY1[Z], actualDir[Z], PRECISION);
      
      dir = c1.getDeclaredField("cameraZ"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene1);
      assertEquals(expectedDirZ1[X], actualDir[X], PRECISION);
      assertEquals(expectedDirZ1[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirZ1[Z], actualDir[Z], PRECISION);
      
      /*
       * Scene turned 90 downward.
       */
      pos = c2.getDeclaredField("cameraPos"); //$NON-NLS-1$
      pos.setAccessible(true);
      
      actualPos = (double[])pos.get(scene2);
      assertEquals(expectedPos[X], actualPos[X], PRECISION);
      assertEquals(expectedPos[Y], actualPos[Y], PRECISION);
      assertEquals(expectedPos[Z], actualPos[Z], PRECISION);
      
      dir = c2.getDeclaredField("cameraX"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene2);
      assertEquals(expectedDirX2[X], actualDir[X], PRECISION);
      assertEquals(expectedDirX2[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirX2[Z], actualDir[Z], PRECISION);
      
      dir = c2.getDeclaredField("cameraY"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene2);
      assertEquals(expectedDirY2[X], actualDir[X], PRECISION);
      assertEquals(expectedDirY2[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirY2[Z], actualDir[Z], PRECISION);
      
      dir = c2.getDeclaredField("cameraZ"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene2);
      assertEquals(expectedDirZ2[X], actualDir[X], PRECISION);
      assertEquals(expectedDirZ2[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirZ2[Z], actualDir[Z], PRECISION);
    } catch (Exception e){
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
  }

  /*
   * Test for constructor Scene(double, double, double, double, double).
   */
  public void testScenedoubledoubledoubledoubledouble() {
    double[] actualPos;
    double[] actualDir;
    double[] expectedPos = {1.0, 1.0, 1.0};
    double[] expectedDirX1 = {0.0, -1.0, 0.0};
    double[] expectedDirY1 = {1.0, 0.0, 0.0};
    double[] expectedDirZ1 = {0.0, 0.0, 1.0};
    double[] expectedDirX2 = {1.0, 0.0, 0.0};
    double[] expectedDirY2 = {0.0, 0.0, -1.0};
    double[] expectedDirZ2 = {0.0, 1.0, 0.0};
    double phi1 = -90.0;
    double rho1 = 0.0;
    double phi2 = 0.0;
    double rho2 = -90.0;
    Scene scene1 = new Scene(expectedPos[X], expectedPos[Y], expectedPos[Z],
                             phi1, rho1);
    Scene scene2 = new Scene(expectedPos[X], expectedPos[Y], expectedPos[Z],
                             phi2, rho2);
    Field pos, dir;
    
    Class c1 = scene1.getClass();
    Class c2 = scene2.getClass();
    
    try {
      /*
       * Scene turned 90 to the right.
       */
      pos = c1.getDeclaredField("cameraPos"); //$NON-NLS-1$
      pos.setAccessible(true);
      
      actualPos = (double[])pos.get(scene1);
      assertEquals(expectedPos[X], actualPos[X], PRECISION);
      assertEquals(expectedPos[Y], actualPos[Y], PRECISION);
      assertEquals(expectedPos[Z], actualPos[Z], PRECISION);
      
      dir = c1.getDeclaredField("cameraX"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene1);
      assertEquals(expectedDirX1[X], actualDir[X], PRECISION);
      assertEquals(expectedDirX1[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirX1[Z], actualDir[Z], PRECISION);
      
      dir = c1.getDeclaredField("cameraY"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene1);
      assertEquals(expectedDirY1[X], actualDir[X], PRECISION);
      assertEquals(expectedDirY1[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirY1[Z], actualDir[Z], PRECISION);
      
      dir = c1.getDeclaredField("cameraZ"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene1);
      assertEquals(expectedDirZ1[X], actualDir[X], PRECISION);
      assertEquals(expectedDirZ1[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirZ1[Z], actualDir[Z], PRECISION);
      
      /*
       * Scene turned 90 downward.
       */
      pos = c2.getDeclaredField("cameraPos"); //$NON-NLS-1$
      pos.setAccessible(true);
      
      actualPos = (double[])pos.get(scene2);
      assertEquals(expectedPos[X], actualPos[X], PRECISION);
      assertEquals(expectedPos[Y], actualPos[Y], PRECISION);
      assertEquals(expectedPos[Z], actualPos[Z], PRECISION);
      
      dir = c2.getDeclaredField("cameraX"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene2);
      assertEquals(expectedDirX2[X], actualDir[X], PRECISION);
      assertEquals(expectedDirX2[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirX2[Z], actualDir[Z], PRECISION);
      
      dir = c2.getDeclaredField("cameraY"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene2);
      assertEquals(expectedDirY2[X], actualDir[X], PRECISION);
      assertEquals(expectedDirY2[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirY2[Z], actualDir[Z], PRECISION);
      
      dir = c2.getDeclaredField("cameraZ"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene2);
      assertEquals(expectedDirZ2[X], actualDir[X], PRECISION);
      assertEquals(expectedDirZ2[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirZ2[Z], actualDir[Z], PRECISION);
    } catch (Exception e){
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
  }

  /*
   * Test for constructor Scene(double, double, double, double, double, double)
   * with invalid argument direction (0;0;0).
   */
  public void testSceneWrongDirectionZero() {
    try {
      Scene scene = new Scene(1.0, 1.0, 1.0, 0.0, 0.0, 0.0);
    } catch (IllegalArgumentException e) {
      return;
    }
    fail("Failed: expected IllegalArgumentException."); //$NON-NLS-1$
  }
  
  public void testAdd() {
    Triangle triangle = new Triangle(1.0, 1.0, 3.0,
                                     1.0, 3.0, 1.0,
                                     3.0, 1.0, 1.0);
    Scene scene = new Scene();
    int size;
    
    size = scene.getTriangles().size();
    
    assertTrue(scene.add(triangle));
    
    assertEquals(size + 1, scene.getTriangles().size());
    assertTrue(scene.getTriangles().contains(triangle));
  }

  public void testGetTriangles() {
    double[] p11 = {1.0, 1.0, 3.0};
    double[] p12 = {1.0, 3.0, 1.0};
    double[] p13 = {3.0, 1.0, 1.0};
    double[] p21 = {10.0, 1.0, 3.0};
    double[] p22 = {10.0, 3.0, 1.0};
    double[] p23 = {13.0, 1.0, 1.0};
    Triangle triangle1 = new Triangle(p11[X], p11[Y], p11[Z],
                                      p12[X], p12[Y], p12[Z],
                                      p13[X], p13[Y], p13[Z]);
    Triangle triangle2 = new Triangle(p21[X], p21[Y], p21[Z],
                                      p22[X], p22[Y], p22[Z],
                                      p23[X], p23[Y], p23[Z]);
    Triangle triangle;
    Scene scene = new Scene();
    Iterator it;
    boolean triangle1OK = false, triangle2OK = false;
    
    scene.add(triangle1);
    scene.add(triangle2);
    
    it = scene.getTriangles().iterator();
    while (it.hasNext()) {
      triangle = (Triangle)it.next();
      if (triangle == triangle1) {
        assertEquals(p11[X], triangle.getPointA()[X], PRECISION);
        assertEquals(p11[Y], triangle.getPointA()[Y], PRECISION);
        assertEquals(p11[Z], triangle.getPointA()[Z], PRECISION);
        assertEquals(p12[X], triangle.getPointB()[X], PRECISION);
        assertEquals(p12[Y], triangle.getPointB()[Y], PRECISION);
        assertEquals(p12[Z], triangle.getPointB()[Z], PRECISION);
        assertEquals(p13[X], triangle.getPointC()[X], PRECISION);
        assertEquals(p13[Y], triangle.getPointC()[Y], PRECISION);
        assertEquals(p13[Z], triangle.getPointC()[Z], PRECISION);
        triangle1OK = true;
      } else if (triangle == triangle2) {
        assertEquals(p21[X], triangle.getPointA()[X], PRECISION);
        assertEquals(p21[Y], triangle.getPointA()[Y], PRECISION);
        assertEquals(p21[Z], triangle.getPointA()[Z], PRECISION);
        assertEquals(p22[X], triangle.getPointB()[X], PRECISION);
        assertEquals(p22[Y], triangle.getPointB()[Y], PRECISION);
        assertEquals(p22[Z], triangle.getPointB()[Z], PRECISION);
        assertEquals(p23[X], triangle.getPointC()[X], PRECISION);
        assertEquals(p23[Y], triangle.getPointC()[Y], PRECISION);
        assertEquals(p23[Z], triangle.getPointC()[Z], PRECISION);
        triangle2OK = true;
      }
    }
    
    assertTrue(triangle1OK);
    assertTrue(triangle2OK);
  }
  
  public void testGetDirection() {
    Scene scene = new Scene();
    double[] result;
    
    result = scene.getDirection();
    
    assertEquals(Scene.DEFAULT_CAMERA_Y[X], result[X], PRECISION);
    assertEquals(Scene.DEFAULT_CAMERA_Y[Y], result[Y], PRECISION);
    assertEquals(Scene.DEFAULT_CAMERA_Y[Z], result[Z], PRECISION);
  }
  
  public void testGetAperture() {
    Scene scene = new Scene();
    
    assertEquals(Scene.DEFAULT_APERTURE, scene.getAperture(), PRECISION);
  }
  
  public void testSetAperture() {
    double expected1 = 1.0;
    double expected2 = 15.0;
    double expected3 = 180.0;
    Scene scene = new Scene();
    
    assertEquals(Scene.DEFAULT_APERTURE, scene.getAperture(), PRECISION);
    scene.setAperture(expected1);
    assertEquals(expected1, scene.getAperture(), PRECISION);
    scene.setAperture(expected2);
    assertEquals(expected2, scene.getAperture(), PRECISION);
    scene.setAperture(expected3);
    assertEquals(expected3, scene.getAperture(), PRECISION);
  }

  public void testSetApertureInvalidLow() {
    double invalid = 0.0;
    Scene scene = new Scene();
    
    try {
      scene.setAperture(invalid);
    } catch (IllegalArgumentException e) {
      return;
    }
    fail("Failed: expected InvalidArgumentException."); //$NON-NLS-1$
  }

  public void testSetApertureInvalidHigh() {
    double invalid = 181.0;
    Scene scene = new Scene();
    
    try {
      scene.setAperture(invalid);
    } catch (IllegalArgumentException e) {
      return;
    }
    fail("Failed: expected InvalidArgumentException."); //$NON-NLS-1$
  }

  public void testSetDirectionCoordHorizontalTurn() {
    double[] refA = {0.0, 1.0, 0.0};
    double[] refB = {-1.0, 0.0, 0.0};
    double[] refC = {0.0, 0.0, 1.0};
    double[] newDirection = {1.0, 0.0, 0.0};
    double[] expectedDirX = {0.0, -1.0, 0.0};
    double[] expectedDirY = {newDirection[X], newDirection[Y], newDirection[Z]};
    double[] expectedDirZ = {0.0, 0.0, 1.0};
    Triangle triangle = new Triangle(1.0, 0.0, 0.0,
                                     0.0, 1.0, 0.0,
                                     0.0, 0.0, 1.0);
    Scene scene = new Scene();
    double[] actualDir;
    Field dir;
    
    scene.add(triangle);
    
    scene.setDirection(newDirection[X], newDirection[Y], newDirection[Z]);

    // Check direction
    Class c = scene.getClass();
    
    try{
      dir = c.getDeclaredField("cameraX"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirX[X], actualDir[X], PRECISION);
      assertEquals(expectedDirX[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirX[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraY"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirY[X], actualDir[X], PRECISION);
      assertEquals(expectedDirY[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirY[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraZ"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirZ[X], actualDir[X], PRECISION);
      assertEquals(expectedDirZ[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirZ[Z], actualDir[Z], PRECISION);
    } catch (Exception e){
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
    
    // Check pointA
    assertEquals(refA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(refA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(refA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(refB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(refB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(refB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(refC[X], triangle.getPointC()[X], PRECISION);
    assertEquals(refC[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(refC[Z], triangle.getPointC()[Z], PRECISION);
  }

  public void testSetDirectionCoordVerticalTurn() {
    double[] refA = {1.0, 0.0, 0.0};
    double[] refB = {0.0, 0.0, 1.0};
    double[] refC = {0.0, -1.0, 0.0};
    double[] newDirection = {0.0, 0.0, -1.0};
    double[] expectedDirX = {1.0, 0.0, 0.0};
    double[] expectedDirY = {newDirection[X], newDirection[Y], newDirection[Z]};
    double[] expectedDirZ = {0.0, 1.0, 0.0};
    Triangle triangle = new Triangle(1.0, 0.0, 0.0,
                                     0.0, 1.0, 0.0,
                                     0.0, 0.0, 1.0);
    Scene scene = new Scene();
    double[] actualDir;
    Field dir;
    
    scene.add(triangle);
    
    scene.setDirection(newDirection[X], newDirection[Y], newDirection[Z]);

    // Check direction
    Class c = scene.getClass();
    
    try{
      dir = c.getDeclaredField("cameraX"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirX[X], actualDir[X], PRECISION);
      assertEquals(expectedDirX[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirX[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraY"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirY[X], actualDir[X], PRECISION);
      assertEquals(expectedDirY[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirY[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraZ"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirZ[X], actualDir[X], PRECISION);
      assertEquals(expectedDirZ[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirZ[Z], actualDir[Z], PRECISION);
    } catch (Exception e){
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
    
    // Check pointA
    assertEquals(refA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(refA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(refA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(refB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(refB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(refB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(refC[X], triangle.getPointC()[X], PRECISION);
    assertEquals(refC[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(refC[Z], triangle.getPointC()[Z], PRECISION);
  }

  public void testSetDirectionCoord() {
    double[] pA = {1.0, 0.0, 0.0};
    double[] pB = {0.0, 1.0, 0.0};
    double[] pC = {0.0, 0.0, 1.0};
    double[] refA = new double[3];
    double[] refB = new double[3];
    double[] refC = new double[3];
    double[] newDirection = {1.0, 1.0, 1.0};
    double[] expectedDirX = {1.0, -1.0, 0.0};
    double[] expectedDirY = {newDirection[X], newDirection[Y], newDirection[Z]};
    double[] expectedDirZ = new double[3];
    double[] temp = new double[3];
    Triangle triangle = new Triangle(pA[X], pA[Y], pA[Z],
                                     pB[X], pB[Y], pB[Z],
                                     pC[X], pC[Y], pC[Z]);
    Scene scene = new Scene();
    double[] actualDir;
    Field dir;
    
    // normalize expectedDirX
    temp[X] = expectedDirX[X];
    temp[Y] = expectedDirX[Y];
    temp[Z] = expectedDirX[Z];
    
    expectedDirX[X] /= Math.sqrt(temp[X] * temp[X] +
                                 temp[Y] * temp[Y] +
                                 temp[Z] * temp[Z]);
    expectedDirX[Y] /= Math.sqrt(temp[X] * temp[X] +
                                 temp[Y] * temp[Y] +
                                 temp[Z] * temp[Z]);
    expectedDirX[Z] /= Math.sqrt(temp[X] * temp[X] +
                                 temp[Y] * temp[Y] +
                                 temp[Z] * temp[Z]);
    
    // normalize expectedDirY
    temp[X] = expectedDirY[X];
    temp[Y] = expectedDirY[Y];
    temp[Z] = expectedDirY[Z];
    
    expectedDirY[X] /= Math.sqrt(temp[X] * temp[X] +
                                 temp[Y] * temp[Y] +
                                 temp[Z] * temp[Z]);
    expectedDirY[Y] /= Math.sqrt(temp[X] * temp[X] +
                                 temp[Y] * temp[Y] +
                                 temp[Z] * temp[Z]);
    expectedDirY[Z] /= Math.sqrt(temp[X] * temp[X] +
                                 temp[Y] * temp[Y] +
                                 temp[Z] * temp[Z]);
    
    // calculate and normalize expectedDirZ
    expectedDirZ[X] = expectedDirX[Y] * expectedDirY[Z] -
                      expectedDirX[Z] * expectedDirY[Y];
    expectedDirZ[Y] = expectedDirX[Z] * expectedDirY[X] -
                      expectedDirX[X] * expectedDirY[Z];
    expectedDirZ[Z] = expectedDirX[X] * expectedDirY[Y] -
                      expectedDirX[Y] * expectedDirY[X];
    
    temp[X] = expectedDirZ[X];
    temp[Y] = expectedDirZ[Y];
    temp[Z] = expectedDirZ[Z];
    
    expectedDirZ[X] /= Math.sqrt(temp[X] * temp[X] +
                                 temp[Y] * temp[Y] +
                                 temp[Z] * temp[Z]);
    expectedDirZ[Y] /= Math.sqrt(temp[X] * temp[X] +
                                 temp[Y] * temp[Y] +
                                 temp[Z] * temp[Z]);
    expectedDirZ[Z] /= Math.sqrt(temp[X] * temp[X] +
                                 temp[Y] * temp[Y] +
                                 temp[Z] * temp[Z]);
    
    // calculate triangle coordinates in expected camera coordinate
    // pointA
    refA[X] = expectedDirX[X] * pA[X] +
              expectedDirX[Y] * pA[Y] +
              expectedDirX[Z] * pA[Z];
    refA[Y] = expectedDirY[X] * pA[X] +
              expectedDirY[Y] * pA[Y] +
              expectedDirY[Z] * pA[Z];
    refA[Z] = expectedDirZ[X] * pA[X] +
              expectedDirZ[Y] * pA[Y] +
              expectedDirZ[Z] * pA[Z];
    // pointB
    refB[X] = expectedDirX[X] * pB[X] +
              expectedDirX[Y] * pB[Y] +
              expectedDirX[Z] * pB[Z];
    refB[Y] = expectedDirY[X] * pB[X] +
              expectedDirY[Y] * pB[Y] +
              expectedDirY[Z] * pB[Z];
    refB[Z] = expectedDirZ[X] * pB[X] +
              expectedDirZ[Y] * pB[Y] +
              expectedDirZ[Z] * pB[Z];
    // pointC
    refC[X] = expectedDirX[X] * pC[X] +
              expectedDirX[Y] * pC[Y] +
              expectedDirX[Z] * pC[Z];
    refC[Y] = expectedDirY[X] * pC[X] +
              expectedDirY[Y] * pC[Y] +
              expectedDirY[Z] * pC[Z];
    refC[Z] = expectedDirZ[X] * pC[X] +
              expectedDirZ[Y] * pC[Y] +
              expectedDirZ[Z] * pC[Z];
    
    scene.add(triangle);
    
    scene.setDirection(newDirection[X], newDirection[Y], newDirection[Z]);

    // Check direction
    Class c = scene.getClass();
    
    try{
      dir = c.getDeclaredField("cameraX"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirX[X], actualDir[X], PRECISION);
      assertEquals(expectedDirX[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirX[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraY"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirY[X], actualDir[X], PRECISION);
      assertEquals(expectedDirY[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirY[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraZ"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirZ[X], actualDir[X], PRECISION);
      assertEquals(expectedDirZ[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirZ[Z], actualDir[Z], PRECISION);
    } catch (Exception e){
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
    
    // Check pointA
    assertEquals(refA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(refA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(refA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(refB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(refB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(refB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(refC[X], triangle.getPointC()[X], PRECISION);
    assertEquals(refC[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(refC[Z], triangle.getPointC()[Z], PRECISION);
  }

  public void testSetDirectionCoordZero() {
    double[] direction = {0.0, 0.0, 0.0};
    Scene scene = new Scene();
    
    try {
      scene.setDirection(direction[X], direction[Y], direction[Z]);
    } catch (IllegalArgumentException e) {
      return;
    }
    fail("Failed: expected IllegalArgumentException."); //$NON-NLS-1$
  }

  public void testSetDirectionAngleHorizontalTurn() {
    double[] refA = {0.0, 1.0, 0.0};
    double[] refB = {-1.0, 0.0, 0.0};
    double[] refC = {0.0, 0.0, 1.0};
    double yawAngle = -90.0;
    double[] expectedDirX = {0.0, -1.0, 0.0};
    double[] expectedDirY = {1.0, 0.0, 0.0};
    double[] expectedDirZ = {0.0, 0.0, 1.0};
    Triangle triangle = new Triangle(1.0, 0.0, 0.0,
                                     0.0, 1.0, 0.0,
                                     0.0, 0.0, 1.0);
    Scene scene = new Scene();
    double[] actualDir;
    Field dir;
    
    scene.add(triangle);
    
    scene.setDirection(yawAngle, 0.0);

    // Check direction
    Class c = scene.getClass();
    
    try{
      dir = c.getDeclaredField("cameraX"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirX[X], actualDir[X], PRECISION);
      assertEquals(expectedDirX[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirX[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraY"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirY[X], actualDir[X], PRECISION);
      assertEquals(expectedDirY[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirY[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraZ"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirZ[X], actualDir[X], PRECISION);
      assertEquals(expectedDirZ[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirZ[Z], actualDir[Z], PRECISION);
    } catch (Exception e){
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
    
    // Check pointA
    assertEquals(refA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(refA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(refA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(refB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(refB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(refB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(refC[X], triangle.getPointC()[X], PRECISION);
    assertEquals(refC[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(refC[Z], triangle.getPointC()[Z], PRECISION);
  }

  public void testSetDirectionAngleVerticalTurn() {
    double[] refA = {1.0, 0.0, 0.0};
    double[] refB = {0.0, 0.0, 1.0};
    double[] refC = {0.0, -1.0, 0.0};
    double pitchAngle = -90.0;
    double[] expectedDirX = {1.0, 0.0, 0.0};
    double[] expectedDirY = {0.0, 0.0, -1.0};
    double[] expectedDirZ = {0.0, 1.0, 0.0};
    Triangle triangle = new Triangle(1.0, 0.0, 0.0,
                                     0.0, 1.0, 0.0,
                                     0.0, 0.0, 1.0);
    Scene scene = new Scene();
    double[] actualDir;
    Field dir;
    
    scene.add(triangle);
    
    scene.setDirection(0.0, pitchAngle);

    // Check direction
    Class c = scene.getClass();
    
    try{
      dir = c.getDeclaredField("cameraX"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirX[X], actualDir[X], PRECISION);
      assertEquals(expectedDirX[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirX[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraY"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirY[X], actualDir[X], PRECISION);
      assertEquals(expectedDirY[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirY[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraZ"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirZ[X], actualDir[X], PRECISION);
      assertEquals(expectedDirZ[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirZ[Z], actualDir[Z], PRECISION);
    } catch (Exception e){
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
    
    // Check pointA
    assertEquals(refA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(refA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(refA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(refB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(refB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(refB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(triangle.getPointC()[X], refC[X], PRECISION);
    assertEquals(triangle.getPointC()[Y], refC[Y], PRECISION);
    assertEquals(triangle.getPointC()[Z], refC[Z], PRECISION);
  }

  public void testSetDirectionAngle() {
    double[] pA = {1.0, 0.0, 0.0};
    double[] pB = {0.0, 1.0, 0.0};
    double[] pC = {0.0, 0.0, 1.0};
    double[] refA = new double[3];
    double[] refB = new double[3];
    double[] refC = new double[3];
    double yawAngle = -30.0;
    double pitchAngle = 60.0;
    double[] expectedDirX = new double[3];
    double[] expectedDirY = new double[3];
    double[] expectedDirZ = new double[3];
    Triangle triangle = new Triangle(pA[X], pA[Y], pA[Z],
                                     pB[X], pB[Y], pB[Z],
                                     pC[X], pC[Y], pC[Z]);
    Scene scene = new Scene();
    double[] actualDir;
    int i;
    Field dir;
    
    for (i = 0; i < 3; ++i) {
      expectedDirX[i] = pA[i];
      expectedDirY[i] = pB[i];
      expectedDirZ[i] = pC[i];
    }
    
    expectedDirX[X] = Math.cos(Math.toRadians(yawAngle));
    expectedDirX[Y] = Math.sin(Math.toRadians(yawAngle));
    
    expectedDirY[X] = -Math.cos(Math.toRadians(pitchAngle)) * Math.sin(Math.toRadians(yawAngle));
    expectedDirY[Y] = Math.cos(Math.toRadians(pitchAngle)) * Math.cos(Math.toRadians(yawAngle));
    expectedDirY[Z] = Math.sin(Math.toRadians(pitchAngle));
    
    expectedDirZ[X] = Math.sin(Math.toRadians(pitchAngle)) * Math.sin(Math.toRadians(yawAngle));
    expectedDirZ[Y] = -Math.sin(Math.toRadians(pitchAngle)) * Math.cos(Math.toRadians(yawAngle));
    expectedDirZ[Z] = Math.cos(Math.toRadians(pitchAngle));
    
    // calculate triangle coordinates in expected camera coordinate
    // pointA
    refA[X] = expectedDirX[X] * pA[X] +
              expectedDirX[Y] * pA[Y] +
              expectedDirX[Z] * pA[Z];
    refA[Y] = expectedDirY[X] * pA[X] +
              expectedDirY[Y] * pA[Y] +
              expectedDirY[Z] * pA[Z];
    refA[Z] = expectedDirZ[X] * pA[X] +
              expectedDirZ[Y] * pA[Y] +
              expectedDirZ[Z] * pA[Z];
    // pointB
    refB[X] = expectedDirX[X] * pB[X] +
              expectedDirX[Y] * pB[Y] +
              expectedDirX[Z] * pB[Z];
    refB[Y] = expectedDirY[X] * pB[X] +
              expectedDirY[Y] * pB[Y] +
              expectedDirY[Z] * pB[Z];
    refB[Z] = expectedDirZ[X] * pB[X] +
              expectedDirZ[Y] * pB[Y] +
              expectedDirZ[Z] * pB[Z];
    // pointC
    refC[X] = expectedDirX[X] * pC[X] +
              expectedDirX[Y] * pC[Y] +
              expectedDirX[Z] * pC[Z];
    refC[Y] = expectedDirY[X] * pC[X] +
              expectedDirY[Y] * pC[Y] +
              expectedDirY[Z] * pC[Z];
    refC[Z] = expectedDirZ[X] * pC[X] +
              expectedDirZ[Y] * pC[Y] +
              expectedDirZ[Z] * pC[Z];
    
    scene.add(triangle);
    
    scene.setDirection(yawAngle, pitchAngle);

    // Check direction
    Class c = scene.getClass();
    
    try{
      dir = c.getDeclaredField("cameraX"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirX[X], actualDir[X], PRECISION);
      assertEquals(expectedDirX[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirX[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraY"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirY[X], actualDir[X], PRECISION);
      assertEquals(expectedDirY[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirY[Z], actualDir[Z], PRECISION);
      
      dir = c.getDeclaredField("cameraZ"); //$NON-NLS-1$
      dir.setAccessible(true);
      
      actualDir = (double[])dir.get(scene);
      assertEquals(expectedDirZ[X], actualDir[X], PRECISION);
      assertEquals(expectedDirZ[Y], actualDir[Y], PRECISION);
      assertEquals(expectedDirZ[Z], actualDir[Z], PRECISION);
    } catch (Exception e){
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
    
    // Check pointA
    assertEquals(refA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(refA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(refA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(refB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(refB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(refB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(refC[X], triangle.getPointC()[X], PRECISION);
    assertEquals(refC[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(refC[Z], triangle.getPointC()[Z], PRECISION);
  }

  public void testSetPos() {
    double[] zero1 = {2.0, 2.0, 2.0};
    double[] zero2 = {1.0, 3.0, 0.0};
    Scene scene = new Scene();
   
    scene.setPos(zero1[X], zero1[Y], zero1[Z]);
    
    assertEquals(zero1[X], scene.getPos()[X], PRECISION);
    assertEquals(zero1[Y], scene.getPos()[Y], PRECISION);
    assertEquals(zero1[Z], scene.getPos()[Z], PRECISION);
    
    scene.turnRight(45.0);
    scene.setPos(zero2[X], zero2[Y], zero2[Z]);
    
    assertEquals(zero2[X], scene.getPos()[X], PRECISION);
    assertEquals(zero2[Y], scene.getPos()[Y], PRECISION);
    assertEquals(zero2[Z], scene.getPos()[Z], PRECISION);
  }

  public void testTurnLeft() {
    double[] p1 = {0.0, 3.0, 1.0};
    double[] p2 = {3.0, 3.0, 2.0};
    double[] p3 = {1.0, 3.0, 4.0};
    double angle = 45.0;
    double rad = Math.toRadians(angle);
    double[] expectedA = new double[3];
    double[] expectedB = new double[3];
    double[] expectedC = new double[3];
    Triangle triangle = new Triangle(p1[X], p1[Y], p1[Z],
                                     p2[X], p2[Y], p2[Z],
                                     p3[X], p3[Y], p3[Z]);
    Scene scene = new Scene();
    
    expectedA[X] = Math.cos(rad) * p1[X] + Math.sin(rad) * p1[Y];
    expectedA[Y] = Math.cos(rad) * p1[Y] - Math.sin(rad) * p1[X];
    expectedA[Z] = p1[Z];
    
    expectedB[X] = Math.cos(rad) * p2[X] + Math.sin(rad) * p2[Y];
    expectedB[Y] = Math.cos(rad) * p2[Y] - Math.sin(rad) * p2[X];
    expectedB[Z] = p2[Z];
    
    expectedC[X] = Math.cos(rad) * p3[X] + Math.sin(rad) * p3[Y];
    expectedC[Y] = Math.cos(rad) * p3[Y] - Math.sin(rad) * p3[X];
    expectedC[Z] = p3[Z];
    
    scene.add(triangle);
    scene.turnLeft(angle);
    
    // Check pointA
    assertEquals(expectedA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(expectedA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(expectedA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(expectedB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(expectedB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(expectedB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(expectedC[X], triangle.getPointC()[X], PRECISION);
    assertEquals(expectedC[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(expectedC[Z], triangle.getPointC()[Z], PRECISION);
  }
  
  public void testTurnRight() {
    double[] p1 = {0.0, 3.0, 1.0};
    double[] p2 = {3.0, 3.0, 2.0};
    double[] p3 = {1.0, 3.0, 4.0};
    double angle = 45.0;
    double rad = Math.toRadians(angle);
    double[] expectedA = new double[3];
    double[] expectedB = new double[3];
    double[] expectedC = new double[3];
    Triangle triangle = new Triangle(p1[X], p1[Y], p1[Z],
                                     p2[X], p2[Y], p2[Z],
                                     p3[X], p3[Y], p3[Z]);
    Scene scene = new Scene();
    
    expectedA[X] = Math.cos(-rad) * p1[X] + Math.sin(-rad) * p1[Y];
    expectedA[Y] = Math.cos(-rad) * p1[Y] - Math.sin(-rad) * p1[X];
    expectedA[Z] = p1[Z];
    
    expectedB[X] = Math.cos(-rad) * p2[X] + Math.sin(-rad) * p2[Y];
    expectedB[Y] = Math.cos(-rad) * p2[Y] - Math.sin(-rad) * p2[X];
    expectedB[Z] = p2[Z];
    
    expectedC[X] = Math.cos(-rad) * p3[X] + Math.sin(-rad) * p3[Y];
    expectedC[Y] = Math.cos(-rad) * p3[Y] - Math.sin(-rad) * p3[X];
    expectedC[Z] = p3[Z];
    
    scene.add(triangle);
    scene.turnRight(angle);
    
    // Check pointA
    assertEquals(expectedA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(expectedA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(expectedA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(expectedB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(expectedB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(expectedB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(expectedC[X], triangle.getPointC()[X], PRECISION);
    assertEquals(expectedC[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(expectedC[Z], triangle.getPointC()[Z], PRECISION);
  }
  
  public void testTurnUpward() {
    double[] p1 = {0.0, 3.0, 1.0};
    double[] p2 = {3.0, 3.0, 2.0};
    double[] p3 = {1.0, 3.0, 4.0};
    double angle = 45.0;
    double rad = Math.toRadians(angle);
    double[] expectedA = new double[3];
    double[] expectedB = new double[3];
    double[] expectedC = new double[3];
    Triangle triangle = new Triangle(p1[X], p1[Y], p1[Z],
                                     p2[X], p2[Y], p2[Z],
                                     p3[X], p3[Y], p3[Z]);
    Scene scene = new Scene();
    
    expectedA[X] = p1[X];
    expectedA[Y] = Math.cos(rad) * p1[Y] + Math.sin(rad) * p1[Z];
    expectedA[Z] = Math.cos(rad) * p1[Z] - Math.sin(rad) * p1[Y];
    
    expectedB[X] = p2[X];
    expectedB[Y] = Math.cos(rad) * p2[Y] + Math.sin(rad) * p2[Z];
    expectedB[Z] = Math.cos(rad) * p2[Z] - Math.sin(rad) * p2[Y];
    
    expectedC[X] = p3[X];
    expectedC[Y] = Math.cos(rad) * p3[Y] + Math.sin(rad) * p3[Z];
    expectedC[Z] = Math.cos(rad) * p3[Z] - Math.sin(rad) * p3[Y];
    
    scene.add(triangle);
    scene.turnUpward(angle);
    
    // Check pointA
    assertEquals(expectedA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(expectedA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(expectedA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(expectedB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(expectedB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(expectedB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(expectedC[X], triangle.getPointC()[X], PRECISION);
    assertEquals(expectedC[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(expectedC[Z], triangle.getPointC()[Z], PRECISION);
  }
  
  public void testTurnDownward() {
    double[] p1 = {0.0, 3.0, 1.0};
    double[] p2 = {3.0, 3.0, 2.0};
    double[] p3 = {1.0, 3.0, 4.0};
    double angle = 45.0;
    double rad = Math.toRadians(angle);
    double[] expectedA = new double[3];
    double[] expectedB = new double[3];
    double[] expectedC = new double[3];
    Triangle triangle = new Triangle(p1[X], p1[Y], p1[Z],
                                     p2[X], p2[Y], p2[Z],
                                     p3[X], p3[Y], p3[Z]);
    Scene scene = new Scene();

    expectedA[X] = p1[X];
    expectedA[Y] = Math.cos(-rad) * p1[Y] + Math.sin(-rad) * p1[Z];
    expectedA[Z] = Math.cos(-rad) * p1[Z] - Math.sin(-rad) * p1[Y];
    
    expectedB[X] = p2[X];
    expectedB[Y] = Math.cos(-rad) * p2[Y] + Math.sin(-rad) * p2[Z];
    expectedB[Z] = Math.cos(-rad) * p2[Z] - Math.sin(-rad) * p2[Y];
    
    expectedC[X] = p3[X];
    expectedC[Y] = Math.cos(-rad) * p3[Y] + Math.sin(-rad) * p3[Z];
    expectedC[Z] = Math.cos(-rad) * p3[Z] - Math.sin(-rad) * p3[Y];
    
    scene.add(triangle);
    scene.turnDownward(angle);
    
    // Check pointA
    assertEquals(expectedA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(expectedA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(expectedA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(expectedB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(expectedB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(expectedB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(expectedC[X], triangle.getPointC()[X], PRECISION);
    assertEquals(expectedC[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(expectedC[Z], triangle.getPointC()[Z], PRECISION);
  }
  
  public void testMoveForward() {
    double[] p1 = {0.0, 3.0, 1.0};
    double[] p2 = {3.0, 3.0, 2.0};
    double[] p3 = {1.0, 3.0, 4.0};
    double step = 1.0;
    double[] expectedA = new double[3];
    double[] expectedB = new double[3];
    double[] expectedC = new double[3];
    Triangle triangle = new Triangle(p1[X], p1[Y], p1[Z],
                                     p2[X], p2[Y], p2[Z],
                                     p3[X], p3[Y], p3[Z]);
    Scene scene = new Scene();
    
    expectedA[X] = p1[X];
    expectedA[Y] = p1[Y] - step;
    expectedA[Z] = p1[Z];
    
    expectedB[X] = p2[X];
    expectedB[Y] = p2[Y] - step;
    expectedB[Z] = p2[Z];
    
    expectedC[X] = p3[X];
    expectedC[Y] = p3[Y] - step;
    expectedC[Z] = p3[Z];
    
    scene.add(triangle);
    scene.moveForward(step);
    
    // Check pointA
    assertEquals(expectedA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(expectedA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(expectedA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(expectedB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(expectedB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(expectedB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(expectedC[X], triangle.getPointC()[X], PRECISION);
    assertEquals(expectedC[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(expectedC[Z], triangle.getPointC()[Z], PRECISION);
  }
  
  public void testMoveForwardToZero() {
    double step = 2.0;
    Scene scene = new Scene(0.0, 0.0, 1.0, 0.0, 0.0, -1.0);
    Field cameraPos;
    double[] pos;
    
    Class c = scene.getClass();
    
    try{
      cameraPos = c.getDeclaredField("cameraPos"); //$NON-NLS-1$
      cameraPos.setAccessible(true);
      pos = (double[])cameraPos.get(scene);
      
      assertTrue(pos[Z] > 0.0);
      assertTrue(pos[Z] - step < 0.0);
      
      scene.moveForward(step);
    
      assertEquals(0.0, pos[Z], PRECISION);
    } catch (Exception e) {
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testMoveBackward() {
    double[] p1 = {0.0, 3.0, 1.0};
    double[] p2 = {3.0, 3.0, 2.0};
    double[] p3 = {1.0, 3.0, 4.0};
    double step = 1.0;
    double[] expectedA = new double[3];
    double[] expectedB = new double[3];
    double[] expectedC = new double[3];
    Triangle triangle = new Triangle(p1[X], p1[Y], p1[Z],
                                     p2[X], p2[Y], p2[Z],
                                     p3[X], p3[Y], p3[Z]);
    Scene scene = new Scene();
    
    expectedA[X] = p1[X];
    expectedA[Y] = p1[Y] + step;
    expectedA[Z] = p1[Z];
    
    expectedB[X] = p2[X];
    expectedB[Y] = p2[Y] + step;
    expectedB[Z] = p2[Z];
    
    expectedC[X] = p3[X];
    expectedC[Y] = p3[Y] + step;
    expectedC[Z] = p3[Z];
    
    scene.add(triangle);
    scene.moveBackward(step);
    
    // Check pointA
    assertEquals(expectedA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(expectedA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(expectedA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(expectedB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(expectedB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(expectedB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(expectedC[X], triangle.getPointC()[X], PRECISION);
    assertEquals(expectedC[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(expectedC[Z], triangle.getPointC()[Z], PRECISION);
  }
  
  public void testMoveBackwardToZero() {
    double step = 2.0;
    Scene scene = new Scene(0.0, 0.0, 1.0, 0.0, 0.0, 1.0);
    Field cameraPos;
    double[] pos;
    
    Class c = scene.getClass();
    
    try{
      cameraPos = c.getDeclaredField("cameraPos"); //$NON-NLS-1$
      cameraPos.setAccessible(true);
      pos = (double[])cameraPos.get(scene);
      
      assertTrue(pos[Z] > 0.0);
      assertTrue(pos[Z] - step < 0.0);
      
      scene.moveBackward(step);
      
      assertEquals(0.0, pos[Z], PRECISION);
    } catch (Exception e) {
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testMoveLeft() {
    double[] p1 = {0.0, 3.0, 1.0};
    double[] p2 = {3.0, 3.0, 2.0};
    double[] p3 = {1.0, 3.0, 4.0};
    double step = 1.0;
    double[] expectedA = new double[3];
    double[] expectedB = new double[3];
    double[] expectedC = new double[3];
    Triangle triangle = new Triangle(p1[X], p1[Y], p1[Z],
                                     p2[X], p2[Y], p2[Z],
                                     p3[X], p3[Y], p3[Z]);
    Scene scene = new Scene();
    
    expectedA[X] = p1[X] + step;
    expectedA[Y] = p1[Y];
    expectedA[Z] = p1[Z];
    
    expectedB[X] = p2[X] + step;
    expectedB[Y] = p2[Y];
    expectedB[Z] = p2[Z];
    
    expectedC[X] = p3[X] + step;
    expectedC[Y] = p3[Y];
    expectedC[Z] = p3[Z];
    
    scene.add(triangle);
    scene.moveLeft(step);
    
    // Check pointA
    assertEquals(expectedA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(expectedA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(expectedA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(expectedB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(expectedB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(expectedB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(expectedC[X], triangle.getPointC()[X], PRECISION);
    assertEquals(expectedC[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(expectedC[Z], triangle.getPointC()[Z], PRECISION);
  }
  
  public void testMoveLeftToZero() {
    double step = 2.0;
    Scene scene = new Scene(0.0, 0.0, 1.0);
    Field cameraPos;
    double[] pos;
    
    scene.turnUpward(90.0);
    scene.turnLeft(90.0);
    
    Class c = scene.getClass();
    
    try{
      cameraPos = c.getDeclaredField("cameraPos"); //$NON-NLS-1$
      cameraPos.setAccessible(true);
      pos = (double[])cameraPos.get(scene);
      
      assertTrue(pos[Z] > 0.0);
      assertTrue(pos[Z] - step < 0.0);
      
      scene.moveLeft(step);
      
      assertEquals(0.0, pos[Z], PRECISION);
    } catch (Exception e) {
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testMoveRight() {
    double[] p1 = {0.0, 3.0, 1.0};
    double[] p2 = {3.0, 3.0, 2.0};
    double[] p3 = {1.0, 3.0, 4.0};
    double step = 1.0;
    double[] expectedA = new double[3];
    double[] expectedB = new double[3];
    double[] expectedC = new double[3];
    Triangle triangle = new Triangle(p1[X], p1[Y], p1[Z],
                                     p2[X], p2[Y], p2[Z],
                                     p3[X], p3[Y], p3[Z]);
    Scene scene = new Scene();
    
    expectedA[X] = p1[X] - step;
    expectedA[Y] = p1[Y];
    expectedA[Z] = p1[Z];
    
    expectedB[X] = p2[X] - step;
    expectedB[Y] = p2[Y];
    expectedB[Z] = p2[Z];
    
    expectedC[X] = p3[X] - step;
    expectedC[Y] = p3[Y];
    expectedC[Z] = p3[Z];
    
    scene.add(triangle);
    scene.moveRight(step);
    
    // Check pointA
    assertEquals(expectedA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(expectedA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(expectedA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(expectedB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(expectedB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(expectedB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(expectedC[X], triangle.getPointC()[X], PRECISION);
    assertEquals(expectedC[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(expectedC[Z], triangle.getPointC()[Z], PRECISION);
  }
  
  public void testMoveRightToZero() {
    double step = 2.0;
    Scene scene = new Scene(0.0, 0.0, 1.0);
    Field cameraPos;
    double[] pos;
    
    scene.turnUpward(90.0);
    scene.turnRight(90.0);
    
    Class c = scene.getClass();
    
    try{
      cameraPos = c.getDeclaredField("cameraPos"); //$NON-NLS-1$
      cameraPos.setAccessible(true);
      pos = (double[])cameraPos.get(scene);
      
      assertTrue(pos[Z] > 0.0);
      assertTrue(pos[Z] - step < 0.0);
      
      scene.moveRight(step);
      
      assertEquals(0.0, pos[Z], PRECISION);
    } catch (Exception e) {
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testMoveUpward() {
    double[] p1 = {0.0, 3.0, 1.0};
    double[] p2 = {3.0, 3.0, 2.0};
    double[] p3 = {1.0, 3.0, 4.0};
    double step = 1.0;
    double[] expectedA = new double[3];
    double[] expectedB = new double[3];
    double[] expectedC = new double[3];
    Triangle triangle = new Triangle(p1[X], p1[Y], p1[Z],
                                     p2[X], p2[Y], p2[Z],
                                     p3[X], p3[Y], p3[Z]);
    Scene scene = new Scene();
    
    expectedA[X] = p1[X];
    expectedA[Y] = p1[Y];
    expectedA[Z] = p1[Z] - step;
    
    expectedB[X] = p2[X];
    expectedB[Y] = p2[Y];
    expectedB[Z] = p2[Z] - step;
    
    expectedC[X] = p3[X];
    expectedC[Y] = p3[Y];
    expectedC[Z] = p3[Z] - step;
    
    scene.add(triangle);
    scene.moveUpward(step);
    
    // Check pointA
    assertEquals(expectedA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(expectedA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(expectedA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(expectedB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(expectedB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(expectedB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(expectedC[X], triangle.getPointC()[X], PRECISION);
    assertEquals(expectedC[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(expectedC[Z], triangle.getPointC()[Z], PRECISION);
  }
  
  public void testMoveUpwardToZero() {
    double step = 2.0;
    Scene scene = new Scene(0.0, 0.0, 1.0);
    Field cameraPos;
    double[] pos;
    
    scene.turnUpward(180.0);
    
    Class c = scene.getClass();
    
    try{
      cameraPos = c.getDeclaredField("cameraPos"); //$NON-NLS-1$
      cameraPos.setAccessible(true);
      pos = (double[])cameraPos.get(scene);
      
      assertTrue(pos[Z] > 0.0);
      assertTrue(pos[Z] - step < 0.0);
      
      scene.moveUpward(step);
      
      assertEquals(0.0, pos[Z], PRECISION);
    } catch (Exception e) {
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testMoveDownward() {
    double[] p1 = {0.0, 3.0, 1.0};
    double[] p2 = {3.0, 3.0, 2.0};
    double[] p3 = {1.0, 3.0, 4.0};
    double step = 1.0;
    double[] expectedA = new double[3];
    double[] expectedB = new double[3];
    double[] expectedC = new double[3];
    Triangle triangle = new Triangle(p1[X], p1[Y], p1[Z],
                                     p2[X], p2[Y], p2[Z],
                                     p3[X], p3[Y], p3[Z]);
    double[] zero = {0.0, 0.0, 2.0};
    Scene scene = new Scene(zero[X], zero[Y], zero[Z]);
    
    expectedA[X] = p1[X];
    expectedA[Y] = p1[Y];
    expectedA[Z] = p1[Z] + step - zero[Z];
    
    expectedB[X] = p2[X];
    expectedB[Y] = p2[Y];
    expectedB[Z] = p2[Z] + step - zero[Z];
    
    expectedC[X] = p3[X];
    expectedC[Y] = p3[Y];
    expectedC[Z] = p3[Z] + step - zero[Z];
    
    scene.add(triangle);
    scene.moveDownward(step);
    
    // Check pointA
    assertEquals(expectedA[X], triangle.getPointA()[X], PRECISION);
    assertEquals(expectedA[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(expectedA[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(expectedB[X], triangle.getPointB()[X], PRECISION);
    assertEquals(expectedB[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(expectedB[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(expectedC[X], triangle.getPointC()[X], PRECISION);
    assertEquals(expectedC[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(expectedC[Z], triangle.getPointC()[Z], PRECISION);
  }

  public void testMoveDownwardToZero() {
    double step = 2.0;
    Scene scene = new Scene(0.0, 0.0, 1.0);
    Field cameraPos;
    double[] pos;
    
    Class c = scene.getClass();
    
    try{
      cameraPos = c.getDeclaredField("cameraPos"); //$NON-NLS-1$
      cameraPos.setAccessible(true);
      pos = (double[])cameraPos.get(scene);
      
      assertTrue(pos[Z] > 0.0);
      assertTrue(pos[Z] - step < 0.0);
      
      scene.moveDownward(step);
      
      assertEquals(0.0, pos[Z], PRECISION);
    } catch (Exception e) {
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testResetScene() {
    Scene scene = new Scene();
    Method resetCamera;
    Field cameraX, cameraY, cameraZ;
    double[] dirX, dirY, dirZ;
    
    Class c = scene.getClass();
    Class[] params = {};
    Object[] args = {};
    
    try {
      resetCamera = c.getDeclaredMethod("resetCamera", params); //$NON-NLS-1$
      resetCamera.setAccessible(true);
      
      cameraX = c.getDeclaredField("cameraX"); //$NON-NLS-1$
      cameraX.setAccessible(true);
      cameraY = c.getDeclaredField("cameraY"); //$NON-NLS-1$
      cameraY.setAccessible(true);
      cameraZ = c.getDeclaredField("cameraZ"); //$NON-NLS-1$
      cameraZ.setAccessible(true);
      
      dirX = (double[])cameraX.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_X[X], dirX[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_X[Y], dirX[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_X[Z], dirX[Z], PRECISION);
      
      dirY = (double[])cameraY.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_Y[X], dirY[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Y[Y], dirY[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Y[Z], dirY[Z], PRECISION);
      
      dirZ = (double[])cameraZ.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_Z[X], dirZ[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Z[Y], dirZ[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Z[Z], dirZ[Z], PRECISION);
      
      scene.turnDownward(45.0);
      scene.turnRight(45.0);
    
      assertTrue(((double[])cameraX.get(scene))[Z] > 0.0);
      assertTrue(((double[])cameraY.get(scene))[Z] < 0.0);
      
      resetCamera.invoke(scene, args);
      
      dirX = (double[])cameraX.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_X[X], dirX[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_X[Y], dirX[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_X[Z], dirX[Z], PRECISION);
      
      dirY = (double[])cameraY.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_Y[X], dirY[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Y[Y], dirY[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Y[Z], dirY[Z], PRECISION);
      
      dirZ = (double[])cameraZ.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_Z[X], dirZ[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Z[Y], dirZ[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Z[Z], dirZ[Z], PRECISION);
      
      scene.turnDownward(45.0);
      scene.turnLeft(45.0);
    
      assertTrue(((double[])cameraX.get(scene))[Z] < 0.0);
      assertTrue(((double[])cameraY.get(scene))[Z] < 0.0);
      
      resetCamera.invoke(scene, args);
      
      dirX = (double[])cameraX.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_X[X], dirX[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_X[Y], dirX[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_X[Z], dirX[Z], PRECISION);
      
      dirY = (double[])cameraY.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_Y[X], dirY[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Y[Y], dirY[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Y[Z], dirY[Z], PRECISION);
      
      dirZ = (double[])cameraZ.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_Z[X], dirZ[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Z[Y], dirZ[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Z[Z], dirZ[Z], PRECISION);
      
      scene.turnUpward(45.0);
      scene.turnLeft(45.0);
    
      assertTrue(((double[])cameraX.get(scene))[Z] > 0.0);
      assertTrue(((double[])cameraY.get(scene))[Z] > 0.0);
      
      resetCamera.invoke(scene, args);
      
      dirX = (double[])cameraX.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_X[X], dirX[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_X[Y], dirX[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_X[Z], dirX[Z], PRECISION);
      
      dirY = (double[])cameraY.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_Y[X], dirY[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Y[Y], dirY[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Y[Z], dirY[Z], PRECISION);
      
      dirZ = (double[])cameraZ.get(scene);
      assertEquals(Scene.DEFAULT_CAMERA_Z[X], dirZ[X], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Z[Y], dirZ[Y], PRECISION);
      assertEquals(Scene.DEFAULT_CAMERA_Z[Z], dirZ[Z], PRECISION);
    } catch (Exception e) {
      fail("Failed: method or field inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testSgn() {
    Scene scene = new Scene();
    double value1 = -2.0;
    double value2 = 0.0;
    double value3 = 2.0;
    int positive = 1;
    int negative = -1;
    Integer result;
    
    Class c = scene.getClass();
    Class[] params = {double.class};
    Object[] args1 = {new Double(value1)};
    Object[] args2 = {new Double(value2)};
    Object[] args3 = {new Double(value3)};
    
    try{
      Method method = c.getDeclaredMethod("sgn", params); //$NON-NLS-1$
      method.setAccessible(true);
      
      result = (Integer)method.invoke(scene, args1);
      assertEquals(negative, result.intValue());
      
      result = (Integer)method.invoke(scene, args2);
      assertEquals(positive, result.intValue());
      
      result = (Integer)method.invoke(scene, args3);
      assertEquals(positive, result.intValue());
    } catch(Exception e){
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testAcos() {
    Scene scene = new Scene();
    double value1 = -2.0;
    double value2 = -1.0;
    double value3 = 0.5;
    double value4 = 0.0;
    double value5 = 1.0;
    double value6 = 2.0;
    Double result;
    
    Class c = scene.getClass();
    Class[] params = {double.class};
    Object[] args1 = {new Double(value1)};
    Object[] args2 = {new Double(value2)};
    Object[] args3 = {new Double(value3)};
    Object[] args4 = {new Double(value4)};
    Object[] args5 = {new Double(value5)};
    Object[] args6 = {new Double(value6)};
    
    try{
      Method method = c.getDeclaredMethod("acos", params); //$NON-NLS-1$
      method.setAccessible(true);
      
      result = (Double)method.invoke(scene, args1);
      assertTrue(Double.isNaN(result.doubleValue()));
      
      result = (Double)method.invoke(scene, args2);
      assertEquals(Math.toDegrees(Math.acos(value2)), result.doubleValue(), PRECISION);
      
      result = (Double)method.invoke(scene, args3);
      assertEquals(Math.toDegrees(Math.acos(value3)), result.doubleValue(), PRECISION);
      
      result = (Double)method.invoke(scene, args4);
      assertEquals(Math.toDegrees(Math.acos(value4)), result.doubleValue(), PRECISION);
      
      result = (Double)method.invoke(scene, args5);
      assertEquals(Math.toDegrees(Math.acos(value5)), result.doubleValue(), PRECISION);
      
      result = (Double)method.invoke(scene, args6);
      assertTrue(Double.isNaN(result.doubleValue()));
    } catch(Exception e){
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testTurnCameraBase() {
    Scene scene = new Scene();
    double[] expectedX1 = {0.0, -1.0, 0.0};
    double[] expectedY1 = {1.0, 0.0, 0.0};
    double[] expectedZ1 = {0.0, 0.0, 1.0};
    double[] expectedX2 = {0.0, -1.0, 0.0};
    double[] expectedY2 = {0.0, 0.0, -1.0};
    double[] expectedZ2 = {1.0, 0.0, 0.0};
    double alpha = 90.0;
    double beta = 90.0;
    Field x, y, z;
    double[] value;
    
    Class c = scene.getClass();
    Class[] params = {double.class, int.class};
    Object[] args1 = {new Double(alpha), new Integer(LEFTRIGHT)};
    Object[] args2 = {new Double(beta), new Integer(UPDOWN)};
    
    try{
      Method method = c.getDeclaredMethod("turnCameraBase", params); //$NON-NLS-1$
      method.setAccessible(true);
      
      x = c.getDeclaredField("cameraX"); //$NON-NLS-1$
      x.setAccessible(true);
      y = c.getDeclaredField("cameraY"); //$NON-NLS-1$
      y.setAccessible(true);
      z = c.getDeclaredField("cameraZ"); //$NON-NLS-1$
      z.setAccessible(true);
      
      method.invoke(scene, args1);
      
      value = (double[])x.get(scene);
      assertEquals(expectedX1[X], value[X], PRECISION);
      assertEquals(expectedX1[Y], value[Y], PRECISION);
      assertEquals(expectedX1[Z], value[Z], PRECISION);
      
      value = (double[])y.get(scene);
      assertEquals(expectedY1[X], value[X], PRECISION);
      assertEquals(expectedY1[Y], value[Y], PRECISION);
      assertEquals(expectedY1[Z], value[Z], PRECISION);
      
      value = (double[])z.get(scene);
      assertEquals(expectedZ1[X], value[X], PRECISION);
      assertEquals(expectedZ1[Y], value[Y], PRECISION);
      assertEquals(expectedZ1[Z], value[Z], PRECISION);
      
      method.invoke(scene, args2);
      
      value = (double[])x.get(scene);
      assertEquals(expectedX2[X], value[X], PRECISION);
      assertEquals(expectedX2[Y], value[Y], PRECISION);
      assertEquals(expectedX2[Z], value[Z], PRECISION);
      
      value = (double[])y.get(scene);
      assertEquals(expectedY2[X], value[X], PRECISION);
      assertEquals(expectedY2[Y], value[Y], PRECISION);
      assertEquals(expectedY2[Z], value[Z], PRECISION);
      
      value = (double[])z.get(scene);
      assertEquals(expectedZ2[X], value[X], PRECISION);
      assertEquals(expectedZ2[Y], value[Y], PRECISION);
      assertEquals(expectedZ2[Z], value[Z], PRECISION);
    } catch(Exception e){
      fail("Failed: field or method inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testTurnCameraBaseInvalidArg() {
    Scene scene = new Scene();
    
    Class c = scene.getClass();
    Class[] params = {double.class, int.class};
    Object[] args = {new Double(15), new Integer(10)};
    
    try{
      Method method = c.getDeclaredMethod("turnCameraBase", params); //$NON-NLS-1$
      method.setAccessible(true);
      
      method.invoke(scene, args);
    } catch (InvocationTargetException ite) {
      if (ite.getTargetException().toString().endsWith("IllegalArgumentException")) //$NON-NLS-1$
        return;
    } catch(Exception e){
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
    fail("Failed: expected IllegalArgumentException."); //$NON-NLS-1$
  }

}
