/*
 * ControlPanel.java
 */
 
package de.ba_ravensburg.gdv.view;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.Border;

import de.ba_ravensburg.gdv.controller.Controller;

/**
 * @author Max Schloss
 */

public class ControlPanel extends JPanel {

  public static final double STEP = 1.0;
  
  private Controller controller;
  
  private JButton btRunter;
  private JButton btHoch;
  private JButton btRechts;
  private JButton btLinks;
  private JButton btZurueck;
  private JButton btVor;
  private JButton btDrehRechts;
  private JButton btDrehLinks;
  private JButton btNickRunter;
  private JButton btNickHoch;
  
  private SpinnerNumberModel smX;
  private SpinnerNumberModel smY;
  private SpinnerNumberModel smZ;
  private SpinnerNumberModel smGier;
  private SpinnerNumberModel smNick;
  private SpinnerNumberModel smZoom;
  
  private JSpinner tfX;
  private JSpinner tfY;
  private JSpinner tfZ;
  private JTextField tfGier;
  private JTextField tfNick;
  private JSpinner tfZoom;
  private JButton btGier;
  
  private JComboBox cbSequenzAuswahl;
  private JButton btStartSeq;
  
  public ControlPanel(Controller theController) {
    super();

    this.setLayout(new GridBagLayout());
    Dimension dim = new Dimension(225, 328);
    this.setPreferredSize(dim);
    this.setMinimumSize(dim);
    this.setMaximumSize(dim);
    controller = theController;
    
    // Panel mit der Steuerung einfgen
    this.add(getSteuerungPanel(), Utils.makeGBC(0, 0, 1, 1, 1.0, 0.0,
                                                GridBagConstraints.NORTHWEST,
                                                GridBagConstraints.HORIZONTAL));

    // Panel mit der Angabe der Position einfgen
    this.add(getPositionPanel(),
             Utils.makeGBC(0, 1, 1, 1, 0.0, 0.0, 10, 0, 0, 0,
                           GridBagConstraints.NORTHWEST,
                           GridBagConstraints.HORIZONTAL));

    // Panel fr automatische Bewegung einfgen
    this.add(getAutoBewegungPanel(),
             Utils.makeGBC(0, 3, 1, 1, 0.0, 0.0, 10, 0, 0, 0,
                           GridBagConstraints.NORTHWEST,
                           GridBagConstraints.HORIZONTAL));
  }
  
  private JPanel getSteuerungPanel() {
    JPanel steuerungPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
    steuerungPanel.setMinimumSize(new Dimension(100, 95));
    steuerungPanel.setBorder(BorderFactory.createTitledBorder(
                             BorderFactory.createEtchedBorder(),
                             "Steuerung")); //$NON-NLS-1$

    Dimension buttDim = new Dimension(20, 20);
    Border border = BorderFactory.createEmptyBorder();
    Font font = new Font("System", Font.PLAIN, 10); //$NON-NLS-1$
    Insets margin = new Insets(1, 1, 1, 1);
    
    
    // Richtungsbuttons
    JPanel bewegungPanel = new JPanel(new GridBagLayout());
    
    btVor = new JButton(new ImageIcon(this.getClass().getResource(
                                        "pics/vor.gif"))); //$NON-NLS-1$
    btVor.setToolTipText("vor"); //$NON-NLS-1$
    btVor.setPreferredSize(buttDim);
    btVor.setBorder(border);
    btVor.setFont(font);
    btVor.setMargin(margin);
    btVor.addActionListener(controller);
    bewegungPanel.add(btVor, Utils.makeGBC(0, 0, 3, 1, 0.0, 0.0, 0, 0, 5, 0,
                                           GridBagConstraints.NORTH,
                                           GridBagConstraints.NONE));
    
    btZurueck = new JButton(new ImageIcon(this.getClass().getResource(
                                            "pics/zurueck.gif"))); //$NON-NLS-1$
    btZurueck.setToolTipText("zurck"); //$NON-NLS-1$
    btZurueck.setPreferredSize(buttDim);
    btZurueck.setBorder(border);
    btZurueck.setFont(font);
    btZurueck.setMargin(margin);
    btZurueck.addActionListener(controller);
    bewegungPanel.add(btZurueck,
                      Utils.makeGBC(0, 2, 3, 1, 0.0, 0.0, 3, 0, 0, 0,
                                    GridBagConstraints.NORTH,
                                    GridBagConstraints.NONE));
    
    btLinks = new JButton(new ImageIcon(this.getClass().getResource(
                                          "pics/links.gif"))); //$NON-NLS-1$
    btLinks.setToolTipText("links"); //$NON-NLS-1$
    btLinks.setPreferredSize(buttDim);
    btLinks.setFont(font);
    btLinks.setBorder(border);
    btLinks.setMargin(margin);
    btLinks.addActionListener(controller);
    bewegungPanel.add(btLinks,
                      Utils.makeGBC(0, 1, 1, 1, 0.0, 0.0, 0, 0, 0, 0,
                                    GridBagConstraints.NORTH,
                                    GridBagConstraints.NONE));
    
    btRechts = new JButton(new ImageIcon(this.getClass().getResource(
                                           "pics/rechts.gif"))); //$NON-NLS-1$
    btRechts.setToolTipText("rechts"); //$NON-NLS-1$
    btRechts.setPreferredSize(buttDim);
    btRechts.setBorder(border);
    btRechts.setFont(font);
    btRechts.setMargin(margin);
    btRechts.addActionListener(controller);
    bewegungPanel.add(btRechts, Utils.makeGBC(2, 1, 1, 1, 0.0, 0.0,
                                              GridBagConstraints.NORTH,
                                              GridBagConstraints.NONE));
              
    JLabel labBewegung = new JLabel("Bewegung"); //$NON-NLS-1$
    labBewegung.setFont(font); //$NON-NLS-1$
    bewegungPanel.add(labBewegung,
                      Utils.makeGBC(1, 1, 1, 1, 0.0, 0.0, 2, 0, 0, 0,
                                    GridBagConstraints.NORTH,
                                    GridBagConstraints.NONE));
              
    // Hhebuttons
    JPanel hoehePanel = new JPanel(new GridBagLayout());  
    
    btHoch = new JButton(new ImageIcon(this.getClass().getResource(
                                         "pics/hoch.gif"))); //$NON-NLS-1$
    btHoch.setToolTipText("hoch"); //$NON-NLS-1$
    btHoch.setPreferredSize(buttDim);
    btHoch.setBorder(border);
    btHoch.setFont(font);
    btHoch.setMargin(margin);
    btHoch.addActionListener(controller);
    hoehePanel.add(btHoch, Utils.makeGBC(0, 0, 1, 1, 0.0, 0.0, 0, 0, 0, 0,
                                         GridBagConstraints.NORTH,
                                         GridBagConstraints.NONE));
              
    btRunter = new JButton(new ImageIcon(this.getClass().getResource(
                                           "pics/runter.gif"))); //$NON-NLS-1$
    btRunter.setToolTipText("runter"); //$NON-NLS-1$
    btRunter.setPreferredSize(buttDim);
    btRunter.setBorder(border);
    btRunter.setFont(font);
    btRunter.setMargin(margin);
    btRunter.addActionListener(controller);
    hoehePanel.add(btRunter, Utils.makeGBC(0, 2, 1, 1, 0.0, 0.0, 0, 0, 0, 0,
                                           GridBagConstraints.NORTH,
                                           GridBagConstraints.NONE));
    
    JLabel labHoehe = new JLabel("Hhe"); //$NON-NLS-1$
    labHoehe.setFont(font); //$NON-NLS-1$
    hoehePanel.add(labHoehe, Utils.makeGBC(0, 1, 1, 1, 0.0, 0.0, 7, 0, 7, 0,
                                           GridBagConstraints.NORTH,
                                           GridBagConstraints.NONE));
    
    // Drehungsbuttons
    JPanel drehungPanel = new JPanel(new GridBagLayout());
    
    btNickHoch = new JButton(new ImageIcon(this.getClass().getResource(
                               "pics/dreh_hoch.gif"))); //$NON-NLS-1$
    btNickHoch.setToolTipText("hoch schauen"); //$NON-NLS-1$
    btNickHoch.setPreferredSize(buttDim);
    btNickHoch.setBorder(border);
    btNickHoch.setFont(font);
    btNickHoch.setMargin(margin);
    btNickHoch.addActionListener(controller);
    drehungPanel.add(btNickHoch, Utils.makeGBC(0, 0, 3, 1, 0.0, 0.0, 0, 0, 5, 0,
                                               GridBagConstraints.NORTH,
                                               GridBagConstraints.NONE));
    
    btNickRunter = new JButton(new ImageIcon(this.getClass().getResource(
                                 "pics/dreh_runter.gif"))); //$NON-NLS-1$
    btNickRunter.setToolTipText("runter schauen"); //$NON-NLS-1$
    btNickRunter.setPreferredSize(buttDim);
    btNickRunter.setBorder(border);
    btNickRunter.setFont(font);
    btNickRunter.setMargin(margin);
    btNickRunter.addActionListener(controller);
    drehungPanel.add(btNickRunter,
                     Utils.makeGBC(0, 2, 3, 1, 0.0, 0.0, 3, 0, 0, 0,
                                   GridBagConstraints.NORTH,
                                   GridBagConstraints.NONE));
    
    btDrehLinks = new JButton(new ImageIcon(this.getClass().getResource(
                                "pics/dreh_links.gif"))); //$NON-NLS-1$
    btDrehLinks.setToolTipText("links drehen"); //$NON-NLS-1$
    btDrehLinks.setPreferredSize(buttDim);
    btDrehLinks.setFont(font);
    btDrehLinks.setBorder(border);
    btDrehLinks.setMargin(margin);
    btDrehLinks.addActionListener(controller);
    drehungPanel.add(btDrehLinks,
                     Utils.makeGBC(0, 1, 1, 1, 0.0, 0.0, 0, 0, 0, 0,
                                   GridBagConstraints.NORTH,
                                   GridBagConstraints.NONE));
    
    btDrehRechts = new JButton(new ImageIcon(this.getClass().getResource(
                                 "pics/dreh_rechts.gif"))); //$NON-NLS-1$
    btDrehRechts.setToolTipText("rechts drehen"); //$NON-NLS-1$
    btDrehRechts.setPreferredSize(buttDim);
    btDrehRechts.setBorder(border);
    btDrehRechts.setFont(font);
    btDrehRechts.setMargin(margin);
    btDrehRechts.addActionListener(controller);
    drehungPanel.add(btDrehRechts, Utils.makeGBC(2, 1, 1, 1, 0.0, 0.0,
                                                 GridBagConstraints.NORTH,
                                                 GridBagConstraints.NONE));
              
    JLabel labDrehung = new JLabel("Drehung"); //$NON-NLS-1$
    labDrehung.setFont(font); //$NON-NLS-1$
    drehungPanel.add(labDrehung, Utils.makeGBC(1, 1, 1, 1, 0.0, 0.0, 2, 0, 0, 0,
                                               GridBagConstraints.NORTH,
                                               GridBagConstraints.NONE));
    
    steuerungPanel.add(bewegungPanel);
    steuerungPanel.add(hoehePanel);
    steuerungPanel.add(drehungPanel);
    
    return steuerungPanel;
  }
  
  private JPanel getPositionPanel() {
    JPanel positionPanel = new JPanel(new GridBagLayout());
    positionPanel.setBorder(BorderFactory.createTitledBorder(
                            BorderFactory.createEtchedBorder(),
                            "Position")); //$NON-NLS-1$

    // Models fr Spinner initialisieren
    smX = new SpinnerNumberModel(0, -500, 500, STEP);
    smY = new SpinnerNumberModel(0, -500, 500, STEP);
    smZ = new SpinnerNumberModel(0, -500, 500, STEP);
    smZoom = new SpinnerNumberModel(1, 1, 179, STEP);
    
    positionPanel.add(new JLabel("X:"), //$NON-NLS-1$
                      Utils.makeGBC(0, 0, 1, 1, 0.0, 0.0, 2, 2, 2, 2,
                                    GridBagConstraints.NORTHWEST,
                                    GridBagConstraints.NONE));
    
    tfX = new JSpinner(smX);
    tfX.addChangeListener(controller);
    positionPanel.add(tfX, Utils.makeGBC(1, 0, 1, 1, 1.0, 0.0, 2, 2, 2, 2,
                                         GridBagConstraints.NORTHWEST,
                                         GridBagConstraints.HORIZONTAL));

    positionPanel.add(new JLabel("Y:"), //$NON-NLS-1$
                      Utils.makeGBC(0, 1, 1, 1, 0.0, 0.0, 2, 2, 2, 2,
                                    GridBagConstraints.NORTHWEST,
                                    GridBagConstraints.NONE));
    
    tfY = new JSpinner(smY);
    tfY.addChangeListener(controller);
    positionPanel.add(tfY, Utils.makeGBC(1, 1, 1, 1, 1.0, 0.0, 2, 2, 2, 2,
                                         GridBagConstraints.NORTHWEST,
                                         GridBagConstraints.HORIZONTAL));
    
    positionPanel.add(new JLabel("Z:"), //$NON-NLS-1$
                      Utils.makeGBC(0, 2, 1, 1, 0.0, 0.0, 2, 2, 2, 2,
                                    GridBagConstraints.NORTHWEST,
                                    GridBagConstraints.NONE));
    
    tfZ = new JSpinner(smZ);
    tfZ.addChangeListener(controller);
    positionPanel.add(tfZ, Utils.makeGBC(1, 2, 1, 1, 1.0, 0.0, 2, 2, 2, 2,
                                         GridBagConstraints.NORTHWEST,
                                         GridBagConstraints.HORIZONTAL));
    
    positionPanel.add(new JLabel("Zoom:"), //$NON-NLS-1$
                      Utils.makeGBC(2, 0, 1, 1, 0.0, 0.0, 2, 10, 2, 2,
                                    GridBagConstraints.NORTHWEST,
                                    GridBagConstraints.NONE));
    
    tfZoom = new JSpinner(smZoom);
    tfZoom.addChangeListener(controller);
    positionPanel.add(tfZoom, Utils.makeGBC(3, 0, 2, 1, 1.0, 0.0, 2, 2, 2, 2,
                                            GridBagConstraints.NORTHWEST,
                                            GridBagConstraints.HORIZONTAL));
                      
    positionPanel.add(new JLabel("Nick:"), //$NON-NLS-1$
                      Utils.makeGBC(2, 1, 1, 1, 0.0, 0.0, 2, 10, 2, 2,
                                    GridBagConstraints.NORTHWEST,
                                    GridBagConstraints.NONE));
    
    tfNick = new JTextField("0"); //$NON-NLS-1$
    positionPanel.add(tfNick, Utils.makeGBC(3, 1, 1, 1, 1.0, 0.0, 2, 2, 2, 2,
                                            GridBagConstraints.NORTHWEST,
                                            GridBagConstraints.HORIZONTAL));
    
    positionPanel.add(new JLabel("Gier:"), //$NON-NLS-1$
                      Utils.makeGBC(2, 2, 1, 1, 0.0, 0.0, 2, 10, 2, 2,
                                    GridBagConstraints.NORTHWEST,
                                    GridBagConstraints.NONE));
    
    tfGier = new JTextField("0"); //$NON-NLS-1$
    positionPanel.add(tfGier, Utils.makeGBC(3, 2, 1, 1, 1.0, 0.0, 2, 2, 2, 2,
                                            GridBagConstraints.NORTHWEST,
                                            GridBagConstraints.HORIZONTAL));
    
    btGier = new JButton("OK"); //$NON-NLS-1$
    btGier.setFont(new Font("System", Font.PLAIN, 10)); //$NON-NLS-1$
    btGier.setMargin(new Insets(1, 1, 1, 1));
    btGier.addActionListener(controller);
    positionPanel.add(btGier, Utils.makeGBC(4, 2, 1, 1, 0.0, 0.0, 2, 2, 2, 2,
                                            GridBagConstraints.NORTHWEST,
                                            GridBagConstraints.NONE));
    return positionPanel;
  }
  
  private JPanel getAutoBewegungPanel() {
    JPanel autoBewegungPanel = new JPanel(new GridBagLayout());
    autoBewegungPanel.setBorder(BorderFactory.createTitledBorder(
                                BorderFactory.createEtchedBorder(),
                                "Automatische Sequenzen")); //$NON-NLS-1$

    cbSequenzAuswahl = new JComboBox(new Object[] {
                                       "Kreisbewegung", //$NON-NLS-1$
                                       "Geradlinige Bewegung" //$NON-NLS-1$
                                     });
    cbSequenzAuswahl.setToolTipText("Bewegungssequenz auswhlen"); //$NON-NLS-1$
    cbSequenzAuswahl.addActionListener(controller);
    autoBewegungPanel.add(cbSequenzAuswahl,
                          Utils.makeGBC(0, 0, 2, 1, 0.0, 0.0, 2, 2, 2, 2,
                                        GridBagConstraints.NORTHWEST,
                                        GridBagConstraints.HORIZONTAL));
              
    JPanel startStopPanel = new JPanel(new FlowLayout(FlowLayout.CENTER, 5, 5));
    btStartSeq = new JButton("Start"); //$NON-NLS-1$
    btStartSeq.setToolTipText("Bewegungssequenz starten"); //$NON-NLS-1$
    btStartSeq.addActionListener(controller);
    startStopPanel.add(btStartSeq);
    
    autoBewegungPanel.add(startStopPanel,
                          Utils.makeGBC(0, 1, 2, 1, 0.0, 0.0, 0, 0, 0, 0,
                                        GridBagConstraints.NORTHWEST,
                                        GridBagConstraints.HORIZONTAL));
              
    return autoBewegungPanel;
  }

  /**
   * @return SpinnerModel for gier-angle
   */
  public SpinnerNumberModel getSmGier() {
    return smGier;
  }

  /**
   * @return SpinnerModel for nick-angle
   */
  public SpinnerNumberModel getSmNick() {
    return smNick;
  }

  /**
   * @return SpinnerModel for x-koordinates
   */
  public SpinnerNumberModel getSmX() {
    return smX;
  }

  /**
   * @return SpinnerModel for y-koordinates
   */
  public SpinnerNumberModel getSmY() {
    return smY;
  }

  /**
   * @return SpinnerModel for z-koordinates
   */
  public SpinnerNumberModel getSmZ() {
    return smZ;
  }

  /**
   * @return SpinnerModel for zoom-angle
   */
  public SpinnerNumberModel getSmZoom() {
    return smZoom;
  }

  /**
   * @return
   */
  public JButton getBtHoch() {
    return btHoch;
  }

  /**
   * @return
   */
  public JButton getBtLinks() {
    return btLinks;
  }

  /**
   * @return
   */
  public JButton getBtRechts() {
    return btRechts;
  }

  /**
   * @return
   */
  public JButton getBtRunter() {
    return btRunter;
  }

  /**
   * @return
   */
  public JButton getBtVor() {
    return btVor;
  }

  /**
   * @return
   */
  public JButton getBtZurueck() {
    return btZurueck;
  }
  
  /**
   * @return
   */
  public JButton getBtStartSeq() {
    return btStartSeq;
  }

  /**
   * @return
   */
  public JButton getBtDrehLinks() {
    return btDrehLinks;
  }

  /**
   * @return
   */
  public JButton getBtDrehRechts() {
    return btDrehRechts;
  }

  /**
   * @return
   */
  public JButton getBtNickHoch() {
    return btNickHoch;
  }

  /**
   * @return
   */
  public JButton getBtNickRunter() {
    return btNickRunter;
  }

  /**
   * @return
   */
  public JComboBox getCbSequenzAuswahl() {
    return cbSequenzAuswahl;
  }

  /**
   * @return
   */
  public JButton getBtGier() {
    return btGier;
  }

  /**
   * @return
   */
  public JTextField getTfGier() {
    return tfGier;
  }

  /**
   * @return
   */
  public JTextField getTfNick() {
    return tfNick;
  }

}
