/*
 * MainUI.java
 */
 
package de.ba_ravensburg.gdv.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Toolkit;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JPanel;

import de.ba_ravensburg.gdv.controller.Controller;

/**
 * @author Max Schloss
 */
public class MainUI extends JFrame {

  private Controller controller;
  private View3DPanel view3dpanel;
  private OverviewPanel overPanel;
  private ControlPanel controlPanel;
  
  public MainUI(Controller theController) {
    super();
    
    controller = theController;
    view3dpanel = new View3DPanel();
    overPanel = new OverviewPanel();
    controlPanel = new ControlPanel(controller);
     
    this.setTitle("3D Simulation"); //$NON-NLS-1$
    this.setSize(792, 581);
    
    Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
    this.setLocation((dim.width - this.getWidth()) / 2,
                     (dim.height - this.getHeight()) / 2);
    this.setContentPane(getJContentPanel());
    this.setVisible(true);
    this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    
    this.addWindowListener(controller);
  }
  
  private JPanel getJContentPanel() {
    JPanel contentPanel = new JPanel(new GridBagLayout());
    
    JPanel borderPanel = new JPanel(new FlowLayout(FlowLayout.CENTER, 0, 0));
    borderPanel.setBorder(BorderFactory.createLineBorder(Color.BLACK, 1));
    borderPanel.add(getView3D());
    
    contentPanel.add(borderPanel,
                     Utils.makeGBC(0, 0, 1, 2, 1.0, 1.0, 5, 5, 5, 5,
                                   GridBagConstraints.NORTHWEST,
                                   GridBagConstraints.BOTH));
    
     JPanel borderPanel2 = new JPanel(new FlowLayout(FlowLayout.CENTER, 0, 0));
     borderPanel2.setBorder(BorderFactory.createLineBorder(Color.BLACK, 1));
     borderPanel2.add(getOverview());
    
    contentPanel.add(borderPanel2,
                     Utils.makeGBC(1, 0, 1, 1, 0.0, 0.0, 10, 0, 0, 0,
                                   GridBagConstraints.NORTH,
                                   GridBagConstraints.NONE));
    
    contentPanel.add(getViewControl(),
                     Utils.makeGBC(1, 1, 1, 1, 0.0, 0.0, 0, 0, 5, 5,
                                   GridBagConstraints.NORTHWEST,
                                   GridBagConstraints.NONE));
    
    return contentPanel;
  }
  
  public View3DPanel getView3D() {
    return view3dpanel;
  }
  
  public OverviewPanel getOverview() {
    return overPanel;
  }
  
  public ControlPanel getViewControl() {
    return controlPanel;
  }

}
